from .classifier_map import classifier_cls_by_name
from .col_ignore_clf import (
    ColumnIgnoringClassifier,
    ObjColIgnoringClassifier,
)

for name in [
    'col_ignore_clf', 'classifier_map', 'name',
]:
    try:
        globals().pop(name)
    except KeyError:
        pass
try:
    del name  # pylint: disable=W0631
except NameError:
    pass



"""Extends scikit-learn tools for hyper-parameter search."""

from sklearn.model_selection import ParameterGrid


class ConstrainedParameterGrid(ParameterGrid):
    """Grid of discrete-valued parameters with constraints.

    Parameters
    ----------
    param_grid : dict of string to sequence, or sequence of such
        The parameter grid to explore, as a dictionary mapping estimator
        parameters to sequences of allowed values.

        An empty dict signifies default parameters.

        A sequence of dicts signifies a sequence of grids to search, and is
        useful to avoid exploring parameter combinations that make no sense
        or have no effect. See the examples below.
    bad_comb : list of dicts, optional
        A list of bad parameter combinations to avoid, each given as a dict.
        All combinations containing a bad combinations will be avoided.

    Example
    -------
    >>> param = {'a': [1, 2], 'b': [3, 4, 5]}
    >>> grid = ConstrainedParameterGrid(param, [{'a': 1, 'b': 4}])
    >>> for params in grid: print(sorted(params.items()))
    [('a', 1), ('b', 3)]
    [('a', 1), ('b', 5)]
    [('a', 2), ('b', 3)]
    [('a', 2), ('b', 4)]
    [('a', 2), ('b', 5)]
    """

    def __init__(self, param_grid, bad_comb=None):
        super().__init__(param_grid)
        self.bad_comb = bad_comb

    def __iter__(self):
        """Iterate over the points in the grid.
        Returns
        -------
        params : iterator over dict of string to any
            Yields dictionaries mapping each estimator parameter to one of its
            allowed values.
        """
        if self.bad_comb is None:
            for params in super().__iter__():
                yield params
        else:
            for params in super().__iter__():
                bad_found = False
                for bad_sub_param in self.bad_comb:
                    if bad_sub_param.items() <= params.items():
                        bad_found = True
                        break
                if not bad_found:
                    yield params



"""Utilities for pandas."""

import skutil.estimators

from ._version import get_versions
__version__ = get_versions()['version']
del get_versions
del _version

for name in ['get_versions', '_version', 'pdutil', 'skutil', 'name']:
    try:
        globals().pop(name)
    except KeyError:
        pass



"""Name to estimator class map."""

import functools
from importlib import import_module


_LIN = 'linear_model'
_SMV = 'svm'
_ENSEMBLE = 'ensemble'
_NAIVE_BAYES = 'naive_bayes'

_CLS_NAME_TO_PARAM_MAP = {
    'LogisticRegression': {
        'module': _LIN,
        'names': ['LogisticRegression', 'logisticregression', 'logreg', 'lr'],
    },
    'LogisticRegressionCV': {
        'module': _LIN,
        'names': ['LogisticRegressionCV', 'logisticregressioncv', 'logregcv',
                  'lrcv'],
    },
    'SGDClassifier': {
        'module': _LIN,
        'names': ['SGDClassifier', 'sgdclassifier', 'sgd'],
    },
    'SVC': {
        'module': _SMV,
        'names': ['SVC', 'svc', 'SVM', 'svm'],
    },
    'RandomForestClassifier': {
        'module': _ENSEMBLE,
        'names': ['RandomForestClassifier', 'randomforestclassifier',
                  'RandomForest', 'randomforest'],
    },
    'BernoulliNB': {
        'module': _NAIVE_BAYES,
        'names': ['BernoulliNB', 'bernoullinb', 'bnb'],
    },
    'GaussianNB': {
        'module': _NAIVE_BAYES,
        'names': ['GaussianNB', 'gaussiannb', 'gnb'],
    },
    'MultinomialNB': {
        'module': _NAIVE_BAYES,
        'names': ['MultinomialNB', 'multinomialnb', 'mnb'],
    },
}

_NAME_TO_MODULE_N_CLS_MAP = {}

for cls_name in _CLS_NAME_TO_PARAM_MAP:
    params = _CLS_NAME_TO_PARAM_MAP[cls_name]
    for name in params['names']:
        _NAME_TO_MODULE_N_CLS_MAP[name] = (params['module'], cls_name)


@functools.lru_cache(maxsize=8)
def _sklearn_submodule(submodule_path):
    return import_module(submodule_path)


def classifier_cls_by_name(name):
    """Get an sklearn classifier class by name.

    Parameters
    ----------
    name : str
        The name of the sklearn classifier class to get. Can also be lower-
        cased. Also, some shorthands are supported (e.g. svm for SVC, logreg
        and lr for LogisticRegression).

    Returns
    -------
    object
        The class object of the desired classifier.

    Example
    -------
    >>> classifier_cls_by_name('LogisticRegression')
    <class 'sklearn.linear_model.logistic.LogisticRegression'>
    >>> classifier_cls_by_name('logisticregression')
    <class 'sklearn.linear_model.logistic.LogisticRegression'>
    >>> classifier_cls_by_name('logreg')
    <class 'sklearn.linear_model.logistic.LogisticRegression'>
    >>> classifier_cls_by_name('lr')
    <class 'sklearn.linear_model.logistic.LogisticRegression'>
    """
    submodule_name, cls_name = _NAME_TO_MODULE_N_CLS_MAP[name]
    submodule_path = 'sklearn.{}'.format(submodule_name)
    submodule = _sklearn_submodule(submodule_path)
    # submodule = getattr(_sklearn(), submodule_name)
    return getattr(submodule, cls_name)



"""Setup for the skutil package."""

# !/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import warnings
import setuptools
import versioneer


# Require Python 3.4 or higher
if sys.version_info.major < 3 or sys.version_info.minor < 5:
    warnings.warn("skutil requires Python 3.5 or higher!")
    sys.exit(1)


INSTALL_REQUIRES = ['numpy', 'scipy', 'scikit-learn', 'decore']
TEST_REQUIRES = ['pytest', 'coverage', 'pytest-cov', 'pandas', 'pdutil']

with open('README.rst') as f:
    README = f.read()

setuptools.setup(
    author="Shay Palachy",
    author_email="shay.palachy@gmail.com",
    name='skutil',
    description='Utilities for scikit-learn.',
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    long_description=README,
    url='https://github.com/shaypal5/skutil',
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=[
        INSTALL_REQUIRES
    ],
    extras_require={
        'test': TEST_REQUIRES + INSTALL_REQUIRES,
    },
    classifiers=[
        # Trove classifiers
        # (https://pypi.python.org/pypi?%3Aaction=list_classifiers)
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Intended Audience :: Developers',
    ],
)
